package lcsb.mapviewer.converter.model.sbgnaf;

import lcsb.mapviewer.common.MimeType;
import lcsb.mapviewer.converter.Converter;
import lcsb.mapviewer.converter.ConverterException;
import lcsb.mapviewer.converter.ConverterParams;
import lcsb.mapviewer.converter.InvalidInputDataExecption;
import lcsb.mapviewer.converter.model.sbgnaf.processors.Processor;
import lcsb.mapviewer.converter.model.sbgnaf.processors.ProcessorFactory;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.model.ModelFullIndexed;
import org.apache.commons.io.FilenameUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.sbgn.SbgnUtil;
import org.sbgn.bindings.Map;
import org.sbgn.bindings.Sbgn;
import org.xml.sax.SAXException;

import javax.xml.bind.JAXBException;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Comparator;
import java.util.Objects;
import java.util.stream.Stream;

public class SBGNAFConverter extends Converter {

  private Logger logger = LogManager.getLogger(SBGNAFConverter.class);

  @Override
  public Model createModel(ConverterParams params) throws InvalidInputDataExecption, ConverterException {
    Model model = new ModelFullIndexed(null);
    if (params.getFilename() != null) {
      model.setName(FilenameUtils.getBaseName(params.getFilename()));
    }
    Sbgn sbgnData = sbgnStream2Sbgn(params.getInputStream());
    Map map = sbgnData.getMap();

    ProcessorFactory processorFactory = new ProcessorFactory(model, sbgnData);

    Stream<Processor> glyphProcessors = map.getGlyph().stream()
        .map(processorFactory::createGlyphProcessor);

    Stream<Processor> arcProcessors = map.getArc().stream()
        .map(processorFactory::createArcProcessor);

    Stream.concat(glyphProcessors, arcProcessors)
        .filter(Objects::nonNull)
        .sorted(Comparator.comparingInt(Processor::getPriority))
        .forEach(Processor::process);

    return model;
  }

  @Override
  public String model2String(Model model) {
    return null;
  }

  @Override
  public String getCommonName() {
    return "SBGN-ML Activity Flow";
  }

  @Override
  public MimeType getMimeType() {
    return MimeType.SBGN;
  }

  @Override
  public String getFileExtension() {
    return "sbgnml";
  }

  /**
   * For a given input stream attempt to create a SBGN object.
   *
   * @param inputStream {@link InputStream} The SBGN data input stream
   * @return {@link Sbgn} The parsed Sbgn object.
   * @throws InvalidInputDataExecption If the input stream is not a valid SBGN format.
   * @throws ConverterException If any other error during conversion occurs.
   */
  private Sbgn sbgnStream2Sbgn(InputStream inputStream) throws InvalidInputDataExecption, ConverterException {
    File file = null;
    try {
      file = this.inputStream2File(inputStream);
      if (!SbgnUtil.isValid(file)) {
        throw new InvalidInputDataExecption("Input format is not valid SBGN format.");
      }
      return SbgnUtil.readFromFile(file);
    } catch (IOException e) {
      throw new ConverterException("Failed to create file from input stream.", e);
    } catch (SAXException | JAXBException e) {
      throw new InvalidInputDataExecption("Input format is not valid SBGN v0.2 format.", e);
    } finally {
      if (file != null && !file.delete()) {
        logger.error("Failed to delete tmp file used for input stream conversion: " + file.getAbsolutePath());
      }
    }
  }
}