package lcsb.mapviewer.converter.model.sbgnaf.processors;

import lcsb.mapviewer.converter.ConverterException;
import lcsb.mapviewer.converter.model.sbgnaf.processors.activityNodes.*;
import lcsb.mapviewer.converter.model.sbgnaf.processors.logicalOperators.AndOperatorProcessor;
import lcsb.mapviewer.converter.model.sbgnaf.processors.logicalOperators.OrOperatorProcessor;
import lcsb.mapviewer.converter.model.sbgnaf.processors.modulatingArcs.NecessaryStimulationProcessor;
import lcsb.mapviewer.converter.model.sbgnaf.processors.modulatingArcs.NegativeInfluenceProcessor;
import lcsb.mapviewer.converter.model.sbgnaf.processors.modulatingArcs.PositiveInfluenceProcessor;
import lcsb.mapviewer.converter.model.sbgnaf.processors.containerNodes.CompartmentProcessor;
import lcsb.mapviewer.converter.model.sbgnaf.processors.modulatingArcs.UnknownInfluenceProcessor;
import lcsb.mapviewer.model.map.model.Model;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.sbgn.ArcClazz;
import org.sbgn.GlyphClazz;
import org.sbgn.bindings.Arc;
import org.sbgn.bindings.Glyph;
import org.sbgn.bindings.Sbgn;

public class ProcessorFactory {

  private Logger logger = LogManager.getLogger(ProcessorFactory.class);

  private Model model;
  private Sbgn sbgnData;

  public ProcessorFactory(Model model, Sbgn sbgnData) {
    this.model = model;
    this.sbgnData = sbgnData;
  }

  public Processor<Glyph> createGlyphProcessor(Glyph glyph) {
    GlyphClazz glyphClazz = GlyphClazz.fromClazz(glyph.getClazz());
    switch (glyphClazz) {
      case PHENOTYPE:
        return new PhenotypeProcessor(glyph, model);
      case BIOLOGICAL_ACTIVITY:
        if (glyph.getGlyph() != null) {
          try {
            return createAuxiliaryUnitProcessor(glyph);
          } catch (ConverterException e) {
            return new BiologicalActivityProcessor(glyph, model);
          }
        }
      case COMPARTMENT:
        return new CompartmentProcessor(glyph, model, sbgnData);
      case AND:
        return new AndOperatorProcessor(glyph, model, sbgnData);
      case OR:
        return new OrOperatorProcessor(glyph, model, sbgnData);
      case NOT:
      case DELAY:
        logger.error(String.format("The '%s' glyph is currently not supported by Minerva.", glyph.getClazz()));
        break;
      default:
        logger.error(String.format("'%s' is not a valid SBGN Activity Flow glyph. Ignoring...", glyph.getClazz()));
    }
    return null;
  }

  public Processor<Arc> createArcProcessor(Arc arc) {
    ArcClazz arcClazz = ArcClazz.fromClazz(arc.getClazz());
    switch (arcClazz) {
      case POSITIVE_INFLUENCE:
        return new PositiveInfluenceProcessor(arc, model);
      case NEGATIVE_INFLUENCE:
        return new NegativeInfluenceProcessor(arc, model);
      case UNKNOWN_INFLUENCE:
        return new UnknownInfluenceProcessor(arc, model);
      case NECESSARY_STIMULATION:
        return new NecessaryStimulationProcessor(arc, model);
      case LOGIC_ARC:
        return null; // this is dealt with by the logical operator processors
      default:
        logger.error(String.format("'%s' is not a valid SBGN Activity Flow arc. Ignoring...", arc.getClazz()));
    }
    return null;
  }

  private Processor<Glyph> createAuxiliaryUnitProcessor(Glyph glyph) throws ConverterException {
    if (glyph.getGlyph().size() != 1) {
      throw new ConverterException(String.format("The biological activity '%s' has an unexpected" +
          "number of sub glyphs. Found: %d | Expected: 1", glyph.toString(), glyph.getGlyph().size()));
    }
    Glyph subGlyph = glyph.getGlyph().get(0);
    if (subGlyph.getEntity() == null) {
      throw new ConverterException("Unexpected format of auxiliary unit.");
    }
    String entityName = subGlyph.getEntity().getName();
    GlyphClazz clazz = GlyphClazz.fromClazz(entityName);
    switch(clazz) {
      case MACROMOLECULE:
        return new MacromoleculeProcessor(glyph, model);
      case SIMPLE_CHEMICAL:
        return new SimpleChemicalProcessor(glyph, model);
      case NUCLEIC_ACID_FEATURE:
        return new NucleicAcidProcessor(glyph, model);
      case UNSPECIFIED_ENTITY:
        return new UnspecifiedProcessor(glyph, model);
      default:
        throw new ConverterException(String.format("The '%s' auxiliary unit is currently not supported by Minerva.",
            glyph.getClazz()));
    }
  }
}
