package lcsb.mapviewer.converter.model.sbgnaf.processors.logicalOperators;

import lcsb.mapviewer.converter.model.sbgnaf.processors.GlyphProcessor;
import lcsb.mapviewer.model.graphics.PolylineData;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.reaction.NodeOperator;
import lcsb.mapviewer.model.map.reaction.Reactant;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.model.map.species.Element;
import org.sbgn.ArcClazz;
import org.sbgn.GlyphClazz;
import org.sbgn.bindings.Arc;
import org.sbgn.bindings.Glyph;
import org.sbgn.bindings.Port;
import org.sbgn.bindings.Sbgn;

import java.awt.geom.Point2D;

abstract class LogicalOperatorProcessor extends GlyphProcessor {

  LogicalOperatorProcessor(GlyphClazz glyphClazz) {
    super(glyphClazz);
  }

  @Override
  public Integer getPriority() {
    return 5;
  }

  /**
   * For a given operator create a {@link Reaction} and add find all incoming logic arcs to the operator. For each such
   * incoming arc create a {@link Reactant} and add it to the {@link Reaction}. The operator itself is provided as
   * an argument and added to the {@link Reaction} as well.
   *
   * Note: The outgoing arc, which also defines the specific type of the {@link Reaction} is handled by a
   * {@link lcsb.mapviewer.converter.model.sbgnaf.processors.ArcProcessor}.
   *
   * @param operator The operator representative for this {@link Reaction}
   * @param glyph The glyph associated with the operator.
   * @param sbgnData The SBGN input data.
   * @param model The Minerva model.
   * @return The generic reaction containing all {@link Reactant} and the specific {@link NodeOperator}.
   */
  Reaction createReactionForOperator(NodeOperator operator, Glyph glyph, Sbgn sbgnData, Model model) {
    Reaction reaction = new Reaction(glyph.getId());
    Port firstPort = glyph.getPort().get(0);
    Port secondPort = glyph.getPort().get(1);

    PolylineData polylineData = new PolylineData(
        new Point2D.Float(firstPort.getX(), firstPort.getY()),
        new Point2D.Float(secondPort.getX(), secondPort.getY()));
    operator.setLine(polylineData);
    reaction.addNode(operator);

    sbgnData
        .getMap()
        .getArc()
        .stream()
        .filter(arc -> ArcClazz.fromClazz(arc.getClazz()) == ArcClazz.LOGIC_ARC)
        .filter(arc -> arc.getTarget().equals(firstPort) || arc.getTarget().equals(secondPort))
        .map(arc -> arc2Reactant(arc, model))
        .forEach(reactant -> {
          reaction.addReactant(reactant);
          operator.addInput(reactant);
        });

    return reaction;
  }

  /**
   * Under the assumption that the given arc is a SBGN AF Logic Arc create a reactant from it.
   *
   * @param arc A Logic Arc.
   * @param model A MINERVA map model.
   * @return The created reactant representing the arc within the model.
   */
  private Reactant arc2Reactant(Arc arc, Model model) {
    Glyph sourceGlyph = (Glyph) arc.getSource();
    Element sourceElement = model.getElementByElementId(sourceGlyph.getId());
    Reactant reactant = new Reactant(sourceElement);
    PolylineData polylineData = arc2PolylineData(arc);
    reactant.setLine(polylineData);
    return reactant;
  }
}
