package lcsb.mapviewer.converter.model.sbgnaf;

import lcsb.mapviewer.converter.*;
import lcsb.mapviewer.model.map.compartment.Compartment;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.reaction.*;
import lcsb.mapviewer.model.map.reaction.type.*;
import lcsb.mapviewer.model.map.species.*;
import org.junit.Ignore;
import org.junit.Test;

import java.io.FileNotFoundException;

import static org.junit.Assert.*;

public class SBGNAFConverterTest {

  private static SBGNAFConverter converter = new SBGNAFConverter();

  private String fileName2TestResourcePath(String filename) {
    return getClass().getResource("/" + filename).getFile();
  }

  @Test
  public void canParseBiologicalActivity() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("simple_sample_activity.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
    assertEquals(1, model.getElements().size());
    assertEquals(0, model.getReactions().size());
    Element element = model.getElements().iterator().next();
    assertTrue(element instanceof GenericProtein);
    assertEquals(model, element.getModel());
    assertNotNull(element.getElementId());
    assertEquals("A", element.getName());
    assertNotNull(element.getZ());
  }

  @Test
  public void canParseSimplePhenotype() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("simple_sample_phenotype.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
    assertEquals(1, model.getElements().size());
    assertEquals(0, model.getReactions().size());
    Element element = model.getElements().iterator().next();
    assertTrue(element instanceof Phenotype);
    assertEquals(model, element.getModel());
    assertNotNull(element.getElementId());
    assertEquals("A", element.getName());
    assertNotNull(element.getZ());
  }

  @Test
  public void canParseSimpleSimpleChemical() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("simple_sample_simple-chemical.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
    assertEquals(1, model.getElements().size());
    assertEquals(0, model.getReactions().size());
    Element element = model.getElements().iterator().next();
    assertTrue(element instanceof SimpleMolecule);
    assertEquals(model, element.getModel());
    assertNotNull(element.getElementId());
    assertEquals("A", element.getName());
    assertNotNull(element.getZ());
  }

  @Test
  public void canParseSimpleUnspecified() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("simple_sample_unspecified.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
    assertEquals(1, model.getElements().size());
    assertEquals(0, model.getReactions().size());
    Element element = model.getElements().iterator().next();
    assertTrue(element instanceof Unknown);
    assertEquals(model, element.getModel());
    assertNotNull(element.getElementId());
    assertEquals("A", element.getName());
    assertNotNull(element.getZ());
  }

  @Test
  public void canParseSimpleNucleicAcid() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("simple_sample_nucleic-acid.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
    assertEquals(1, model.getElements().size());
    assertEquals(0, model.getReactions().size());
    Element element = model.getElements().iterator().next();
    assertTrue(element instanceof Rna);
    assertEquals(model, element.getModel());
    assertNotNull(element.getElementId());
    assertEquals("A", element.getName());
    assertNotNull(element.getZ());
  }

  @Test
  public void canParseSimpleMacromolecule() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("simple_sample_macromolecule.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
    assertEquals(1, model.getElements().size());
    assertEquals(0, model.getReactions().size());
    Element element = model.getElements().iterator().next();
    assertTrue(element instanceof Protein);
    assertEquals(model, element.getModel());
    assertNotNull(element.getElementId());
    assertEquals("A", element.getName());
    assertNotNull(element.getZ());
  }

  @Test
  public void canParseSimpleAnd() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("simple_sample_and.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
    assertEquals(0, model.getElements().size());
    assertEquals(1, model.getReactions().size());
    Reaction reaction = model.getReactions().iterator().next();
    assertEquals(1, reaction.getNodes().size());
    assertTrue(reaction.getNodes().get(0) instanceof AndOperator);
    assertEquals(model, reaction.getModel());
    assertNotNull(reaction.getElementId());
  }

  @Test
  public void canParseComplexAnd() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("complex_sample_and.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
    assertEquals(3, model.getElements().size());
    assertEquals(1, model.getReactions().size());
    Reaction reaction = model.getReactions().iterator().next();
    assertEquals(1, reaction.getOperators().size());
    NodeOperator operator = reaction.getOperators().get(0);
    assertEquals(2, operator.getInputs().size());
    assertEquals(1, operator.getOutputs().size());
    assertTrue(operator.isReactantOperator());
    assertEquals(2, reaction.getReactants().size());
    assertEquals(1, reaction.getProducts().size());
  }

  @Test
  public void canParseSimpleOr() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("simple_sample_or.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
    assertEquals(0, model.getElements().size());
    assertEquals(1, model.getReactions().size());
    Reaction reaction = model.getReactions().iterator().next();
    assertEquals(1, reaction.getNodes().size());
    assertTrue(reaction.getNodes().get(0) instanceof OrOperator);
    assertEquals(model, reaction.getModel());
    assertNotNull(reaction.getElementId());
  }

  @Ignore("To be implemented.") @Test
  public void canParseSimpleNot() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("simple_sample_not.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
  }

  @Ignore("To be implemented.") @Test
  public void canParseSimpleDelay() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("simple_sample_delay.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
  }

  @Test
  public void canParseSimplePositiveInfluence() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("simple_sample_positive-influence.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
    assertEquals(2, model.getElements().size());
    assertEquals(1, model.getReactions().size());
    Reaction reaction = model.getReactions().iterator().next();
    assertTrue(reaction instanceof PositiveInfluenceReaction);
    assertEquals(1, reaction.getReactants().size());
    Element reactant = reaction.getReactants().get(0).getElement();
    assertEquals(reactant, model.getElementByElementId(reactant.getElementId()));
    assertEquals(1, reaction.getProducts().size());
    Element product = reaction.getProducts().get(0).getElement();
    assertEquals(product, model.getElementByElementId(product.getElementId()));
    assertEquals(model, reaction.getModel());
    assertNotNull(reaction.getElementId());
    assertNotNull(reaction.getZ());
    assertNotNull(reaction.getLine());
    assertTrue(reaction.getReactants().stream().allMatch(r -> r.getLine() != null));
    assertTrue(reaction.getProducts().stream().allMatch(p -> p.getLine() != null));
    assertEquals(2, reaction.getLine().getPoints().size());
  }

  @Test
  public void canParseComplexPositiveInfluence() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("complex_sample_positive-influence.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
    assertEquals(4, model.getElements().size());
    assertEquals(3, model.getReactions().size());
    model.getReactions().forEach(reaction -> {
      assertTrue(reaction instanceof PositiveInfluenceReaction);
      assertEquals(1, reaction.getReactants().size());
      assertEquals(1, reaction.getProducts().size());
      assertEquals("Z", reaction.getProducts().get(0).getElement().getName());
    });
  }

  @Test
  public void canParseSimpleNegativeInfluence() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("simple_sample_negative-influence.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
    assertEquals(2, model.getElements().size());
    assertEquals(1, model.getReactions().size());
    Reaction reaction = model.getReactions().iterator().next();
    assertTrue(reaction instanceof NegativeInfluenceReaction);
    assertEquals(1, reaction.getReactants().size());
    Element reactant = reaction.getReactants().get(0).getElement();
    assertEquals(reactant, model.getElementByElementId(reactant.getElementId()));
    assertEquals(1, reaction.getProducts().size());
    Element product = reaction.getProducts().get(0).getElement();
    assertEquals(product, model.getElementByElementId(product.getElementId()));
    assertEquals(model, reaction.getModel());
    assertNotNull(reaction.getElementId());
    assertNotNull(reaction.getZ());
    assertNotNull(reaction.getLine());
    assertTrue(reaction.getReactants().stream().allMatch(r -> r.getLine() != null));
    assertTrue(reaction.getProducts().stream().allMatch(p -> p.getLine() != null));
    assertEquals(2, reaction.getLine().getPoints().size());
  }

  @Test
  public void canParseSimpleUnknownInfluence() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("simple_sample_unknown-influence.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
    assertEquals(2, model.getElements().size());
    assertEquals(1, model.getReactions().size());
    Reaction reaction = model.getReactions().iterator().next();
    assertTrue(reaction instanceof ReducedModulationReaction);
    assertEquals(1, reaction.getReactants().size());
    Element reactant = reaction.getReactants().get(0).getElement();
    assertEquals(reactant, model.getElementByElementId(reactant.getElementId()));
    assertEquals(1, reaction.getProducts().size());
    Element product = reaction.getProducts().get(0).getElement();
    assertEquals(product, model.getElementByElementId(product.getElementId()));
    assertEquals(model, reaction.getModel());
    assertNotNull(reaction.getElementId());
    assertNotNull(reaction.getZ());
    assertNotNull(reaction.getLine());
    assertTrue(reaction.getReactants().stream().allMatch(r -> r.getLine() != null));
    assertTrue(reaction.getProducts().stream().allMatch(p -> p.getLine() != null));
    assertEquals(2, reaction.getLine().getPoints().size());
  }

  @Test
  public void canParseSimpleNecessaryStimulation() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("simple_sample_necessary-stimulation.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
    assertEquals(2, model.getElements().size());
    assertEquals(1, model.getReactions().size());
    Reaction reaction = model.getReactions().iterator().next();
    assertTrue(reaction instanceof ReducedTriggerReaction);
    assertEquals(1, reaction.getReactants().size());
    Element reactant = reaction.getReactants().get(0).getElement();
    assertEquals(reactant, model.getElementByElementId(reactant.getElementId()));
    assertEquals(1, reaction.getProducts().size());
    Element product = reaction.getProducts().get(0).getElement();
    assertEquals(product, model.getElementByElementId(product.getElementId()));
    assertEquals(model, reaction.getModel());
    assertNotNull(reaction.getElementId());
    assertNotNull(reaction.getZ());
    assertNotNull(reaction.getLine());
    assertTrue(reaction.getReactants().stream().allMatch(r -> r.getLine() != null));
    assertTrue(reaction.getProducts().stream().allMatch(p -> p.getLine() != null));
    assertEquals(2, reaction.getLine().getPoints().size());
  }

  @Test
  public void canParseSimpleCompartment() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("simple_sample_compartment.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
    assertEquals(1, model.getCompartments().size());
    Compartment compartment = model.getCompartments().get(0);
    assertEquals(0, compartment.getElements().size());
    assertEquals("A", compartment.getName());
    assertNotNull(compartment.getZ());
  }

  @Test
  public void canParseComplexCompartment() throws FileNotFoundException, InvalidInputDataExecption, ConverterException {
    String resource = fileName2TestResourcePath("complex_sample_compartment.sbgnml");
    ConverterParams params = new ConverterParams().filename(resource);
    Model model = converter.createModel(params);
    assertEquals(3, model.getElements().size());
    assertEquals(1, model.getCompartments().size());
    Compartment compartment = model.getCompartments().get(0);
    assertEquals(2, compartment.getElements().size());
    assertEquals("C", compartment.getName());
    assertNotNull(compartment.getZ());
  }
}
